////////////////////////////////////////
// Programming project code
// UniBw M, 2022, 2023, 2024
// www.unibw.de/inf2
// (c) Mark Minas (mark.minas@unibw.de)
////////////////////////////////////////

package pp.monopoly.client;

import java.lang.System.Logger;
import java.lang.System.Logger.Level;
import java.util.prefs.Preferences;

import com.jme3.app.Application;
import com.jme3.app.state.AbstractAppState;
import com.jme3.app.state.AppStateManager;
import com.jme3.asset.AssetLoadException;
import com.jme3.asset.AssetNotFoundException;
import com.jme3.audio.AudioData;
import com.jme3.audio.AudioNode;

import pp.monopoly.notification.GameEventListener;
import pp.monopoly.notification.SoundEvent;
import static pp.util.PreferencesUtils.getPreferences;

/**
 * An application state that plays sounds.
 */
public class GameSound extends AbstractAppState implements GameEventListener {
    private static final Logger LOGGER = System.getLogger(GameSound.class.getName());
    private static final Preferences PREFERENCES = getPreferences(GameSound.class);
    private static final String ENABLED_PREF = "enabled"; //NON-NLS

    private AudioNode passStartSound;
    private AudioNode eventCardSound;
    private AudioNode gulagSound;
    private AudioNode diceRollSound;
    private AudioNode moneyCollectSound;
    private AudioNode moneyLostSound;
    private AudioNode tradeAcceptedSound;
    private AudioNode tradeRejectedSound;
    private AudioNode winnerSound;
    private AudioNode looserSound;
    private AudioNode buttonSound;

    /**
     * Checks if sound is enabled in the preferences.
     *
     * @return {@code true} if sound is enabled, {@code false} otherwise.
     */
    public static boolean enabledInPreferences() {
        return PREFERENCES.getBoolean(ENABLED_PREF, true);
    }

    /**
     * Toggles the game sound on or off.
     */
    public void toggleSound() {
        setEnabled(!isEnabled());
    }

    /**
     * Sets the enabled state of this AppState.
     * Overrides {@link com.jme3.app.state.AbstractAppState#setEnabled(boolean)}
     *
     * @param enabled {@code true} to enable the AppState, {@code false} to disable it.
     */
    @Override
    public void setEnabled(boolean enabled) {
        if (isEnabled() == enabled) return;
        super.setEnabled(enabled);
        LOGGER.log(Level.INFO, "Sound enabled: {0}", enabled); //NON-NLS
        PREFERENCES.putBoolean(ENABLED_PREF, enabled);
    }

    /**
     * Initializes the sound effects for the game.
     * Overrides {@link AbstractAppState#initialize(AppStateManager, Application)}
     *
     * @param stateManager The state manager
     * @param app          The application
     */
    @Override
    public void initialize(AppStateManager stateManager, Application app) {
        super.initialize(stateManager, app);
        passStartSound = loadSound(app, "Sound/Effects/passStart.ogg");
        eventCardSound = loadSound(app, "Sound/Effects/eventCard.ogg");
        gulagSound = loadSound(app, "Sound/Effects/gulag.ogg");
        diceRollSound = loadSound(app, "Sound/Effects/diceRoll.ogg");
        moneyCollectSound = loadSound(app, "Sound/Effects/moneyCollect.ogg");
        moneyLostSound = loadSound(app, "Sound/Effects/moneyLost.ogg");
        tradeAcceptedSound = loadSound(app, "Sound/Effects/tradeAccepted.ogg");
        tradeRejectedSound = loadSound(app, "Sound/Effects/tradeRejected.ogg");
        winnerSound = loadSound(app, "Sound/Effects/winner.ogg");
        looserSound = loadSound(app, "Sound/Effects/looser.ogg");
        buttonSound = loadSound(app, "Sound/Effects/button.ogg");
    }

    /**
     * Loads a sound from the specified file.
     *
     * @param app  The application
     * @param name The name of the sound file.
     * @return The loaded AudioNode.
     */
    private AudioNode loadSound(Application app, String name) {
        try {
            final AudioNode sound = new AudioNode(app.getAssetManager(), name, AudioData.DataType.Buffer);
            sound.setLooping(false);
            sound.setPositional(false);
            return sound;
        }
        catch (AssetLoadException | AssetNotFoundException ex) {
            LOGGER.log(Level.ERROR, ex.getMessage(), ex);
        }
        return null;
    }

    /**
     * Plays the passStart sound effect.
     */
    public void passStart() {
        if (isEnabled() && passStartSound != null)
            passStartSound.playInstance();
    }
    /**
     * Plays the eventCard sound effect.
     */
    public void eventCard() {
        if (isEnabled() && eventCardSound != null)
            eventCardSound.playInstance();
    }
    /**
     * Plays the gulag sound effect.
     */
    public void gulag() {
        if (isEnabled() && gulagSound != null)
            gulagSound.playInstance();
    }
    /**
     * Plays the diceRoll sound effect.
     */
    public void diceRoll() {
        if (isEnabled() && diceRollSound != null)
            diceRollSound.playInstance();
    }
    /**
     * Plays the moneyCollect sound effect.
     */
    public void moneyCollect() {
        if (isEnabled() && moneyCollectSound != null)
            moneyCollectSound.playInstance();
    }
    /**
     * Plays the moneyLost sound effect.
     */
    public void moneyLost() {
        if (isEnabled() && moneyLostSound != null)
            moneyLostSound.playInstance();
    }
    /**
     * Plays the tradeAccepted sound effect.
     */
    public void tradeAccepted() {
        if (isEnabled() && tradeAcceptedSound != null)
            tradeAcceptedSound.playInstance();
    }
    /**
     * Plays the tradeRejected sound effect.
     */
    public void tradeRejected() {
        if (isEnabled() && tradeRejectedSound != null)
            tradeRejectedSound.playInstance();
    }
    /**
     * Plays the winner sound effect.
     */
    public void winner() {
        if (isEnabled() && winnerSound != null)
            winnerSound.playInstance();
    }
    /**
     * Plays the looser sound effect.
     */
    public void looser() {
        if (isEnabled() && looserSound != null)
            looserSound.playInstance();
    }
    /**
     * Plays the button sound effect.
     */
    public void button() {
        if (isEnabled() && buttonSound != null)
            buttonSound.playInstance();
    }

    @Override
    public void receivedEvent(SoundEvent event) {
        switch (event.sound()) {
            case PASS_START -> passStart();
            case EVENT_CARD -> eventCard();
            case GULAG -> eventCard();
            case DICE_ROLL -> eventCard();
            case MONEY_COLLECTED -> eventCard();
            case MONEY_LOST -> eventCard();
            case TRADE_ACCEPTED -> eventCard();
            case TRADE_REJECTED -> eventCard();
            case WINNER -> eventCard();
            case LOSER -> eventCard();
            case BUTTON -> eventCard();
        }
    }
}
