package pp.monopoly.client;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import com.jme3.app.SimpleApplication;
import com.jme3.font.BitmapFont;
import com.jme3.font.BitmapText;
import com.jme3.input.KeyInput;
import com.jme3.input.controls.ActionListener;
import com.jme3.input.controls.KeyTrigger;
import com.jme3.system.AppSettings;
import com.simsilica.lemur.GuiGlobals;
import com.simsilica.lemur.style.BaseStyles;

import pp.dialog.DialogBuilder;
import pp.dialog.DialogManager;
import pp.graphics.Draw;
import pp.monopoly.client.gui.*;
import pp.monopoly.client.gui.popups.BuyCard;
import pp.monopoly.game.client.ClientGameLogic;
import pp.monopoly.game.client.MonopolyClient;
import pp.monopoly.game.client.ServerConnection;
import pp.monopoly.notification.GameEventListener;
import pp.monopoly.notification.InfoTextEvent;
import pp.monopoly.server.MonopolyServer;

public class MonopolyApp extends SimpleApplication implements MonopolyClient, GameEventListener {

    private BitmapText topText;
    private final ServerConnection serverConnection;
    private final ClientGameLogic logic;
    private final MonopolyAppConfig config;
    private final ActionListener escapeListener = (name, isPressed, tpf) -> handleEscape(isPressed);
    private final DialogManager dialogManager = new DialogManager(this);
    private final ExecutorService executor = Executors.newCachedThreadPool();
    private final Draw draw;
    private SettingsMenu settingsMenu;
    private TestWorld testWorld;
    private boolean isSettingsMenuOpen = false;
    private boolean inputBlocked = false;
    private MonopolyServer monopolyServer;
    private NetworkSupport networkSupport;

    // TODO Temp später entfernen

    private BuildingProperty buildingProperty;
    private FoodField foodField;
    private GateField gateField;
    private BuyCard buyCard;
    private boolean isBuyCardPopupOpen = false;
    private final ActionListener BListener = (name, isPressed, tpf) -> handleB(isPressed);

    /**
     * Path to the styles script for GUI elements.
     */
    private static final String STYLES_SCRIPT = "Interface/Lemur/pp-styles.groovy"; // NON-NLS
    /**
     * Path to the font resource used in the GUI.
     */
    private static final String FONT = "Interface/Fonts/Default.fnt"; // NON-NLS

    public static void main(String[] args) {
        new MonopolyApp().start();
    }

    public MonopolyApp() {
        this.draw = new Draw(assetManager);
        config = new MonopolyAppConfig();
        networkSupport = new NetworkSupport(this); // Initialize NetworkSupport
        serverConnection = networkSupport;
        logic = new ClientGameLogic(serverConnection);
        logic.addListener(this);
        setShowSettings(config.getShowSettings());
        setSettings(makeSettings());
    }

    @Override
    public MonopolyAppConfig getConfig() {
        return config;
    }

    @Override
    public ClientGameLogic getGameLogic() {
        return logic;
    }

    public NetworkSupport getNetworkSupport() {
        return networkSupport;
    }

    private AppSettings makeSettings() {
        final AppSettings settings = new AppSettings(true);
        settings.setTitle("Monopoly Game");
        settings.setResolution(config.getResolutionWidth(), config.getResolutionHeight());
        settings.setFullscreen(config.fullScreen());
        return settings;
    }

    @Override
    public void simpleInitApp() {
        GuiGlobals.initialize(this);
        BaseStyles.loadStyleResources(STYLES_SCRIPT);
        GuiGlobals.getInstance().getStyles().setDefaultStyle("pp"); // NON-NLS
        final BitmapFont normalFont = assetManager.loadFont(FONT); // NON-NLS

        setupInput();
        setupGui();

        // Zeige das Startmenü
        StartMenu.createStartMenu(this);
    }

    private void setupGui() {
        BitmapFont normalFont = assetManager.loadFont("Interface/Fonts/Default.fnt");
        topText = new BitmapText(normalFont);
        topText.setLocalTranslation(10, settings.getHeight() - 10, 0);
        guiNode.attachChild(topText);
    }

    private void setupInput() {
        inputManager.deleteMapping(INPUT_MAPPING_EXIT);
        inputManager.setCursorVisible(true);
        inputManager.addMapping("ESC", new KeyTrigger(KeyInput.KEY_ESCAPE));
        inputManager.addListener(escapeListener, "ESC");

        inputManager.addMapping("B", new KeyTrigger(KeyInput.KEY_B));
        inputManager.addListener(BListener, "B");
    }

    private void handleEscape(boolean isPressed) {
        if (isPressed) {
            if (settingsMenu != null && isSettingsMenuOpen) {
                // Schließe das SettingsMenu
                System.out.println("Schließe SettingsMenu...");
                settingsMenu.close();
                settingsMenu = null;
                setSettingsMenuOpen(false);
            } else {
                // Öffne das SettingsMenu
                System.out.println("Öffne SettingsMenu...");
                settingsMenu = new SettingsMenu(this);
                settingsMenu.open();
                setSettingsMenuOpen(true);
            }
        }
    }

    //logik zum wechselnden erscheinen und verschwinden beim drücken von B //TODO süäter entfernen
    private void handleB(boolean isPressed) {
        if (isPressed) {
            if (gateField != null && isBuyCardPopupOpen) {
                // Schließe das SettingsMenu
                System.out.println("Schließe BuyCardPopup...");
                gateField.close();
                gateField = null;
                setBuyCardPopupOpen(false);
            } else {
                // Öffne das SettingsMenu
                System.out.println("Öffne BuyCardPopup...");
                gateField = new GateField(this);
                gateField.open();
                setBuyCardPopupOpen(true);
            }
        }
    }

    private void blockInputs() {
        if (!inputBlocked) {
            System.out.println("Blockiere Eingaben...");
            inputManager.setCursorVisible(true); // Cursor sichtbar machen
            inputManager.clearMappings(); // Alle Mappings entfernen
            inputBlocked = true;
        }
    }

    public void unblockInputs() {
        if (inputBlocked) {
            System.out.println("Aktiviere Eingaben...");
            setupInput(); // Standard-Eingaben neu registrieren
            inputBlocked = false;
        }
    }

    public void setInfoText(String text) {
        topText.setText(text);
    }

    @Override
    public void receivedEvent(InfoTextEvent event) {
        setInfoText(event.key());
    }

    @Override
    public void stop(boolean waitFor) {
        if (executor != null) executor.shutdownNow();
        serverConnection.disconnect();
        super.stop(waitFor);
    }

    public DialogManager getDialogManager() {
        return dialogManager;
    }

    public Draw getDraw() {
        return draw;
    }

    public ExecutorService getExecutor() {
        return executor;
    }

    public void closeApp() {
        stop();
    }

    public void errorDialog(String errorMessage) {
        DialogBuilder.simple(dialogManager)
                .setTitle("Fehler")
                .setText(errorMessage)
                .setOkButton("OK")
                .build()
                .open();
    }

    public void setSettingsMenuOpen(boolean isOpen) {
        this.isSettingsMenuOpen = isOpen;
    }

    // TODO später entfernen

    public void setBuyCardPopupOpen(boolean isOpen) {
        this.isBuyCardPopupOpen = isOpen;
    }

    @Override
    public void simpleUpdate(float tpf) {
        if (testWorld != null) {
            testWorld.update(tpf); // Aktualisiere die Kamera in der TestWorld
        }
    }

    public void startTestWorld() {
        guiNode.detachAllChildren(); // Entferne GUI
        testWorld = new TestWorld(this); // Erstelle eine Instanz von TestWorld
        testWorld.initializeScene(); // Initialisiere die Szene
    }

    // TODO später entfernen

    public void startBuyCard() {
    }

    public void returnToMenu() {
        guiNode.detachAllChildren(); // Entferne die GUI
        StartMenu.createStartMenu(this); // Zeige das Startmenü erneut
    }

    /**
     * Startet den Server in einem neuen Thread.
     */
    public void startServer() {
        new Thread(() -> {
            try {
                MonopolyServer.main(new String[0]); // Startet den MonopolyServer
            } catch (Exception e) {
                errorDialog("Fehler: Server konnte nicht gestartet werden.");
            }
        }).start();
    }

    public MonopolyServer getMonopolyServer() {
        return monopolyServer;
    }

    public ServerConnection getServerConnection() {
        return serverConnection;
    }
}
