package pp.monopoly.client.gui;

import com.jme3.material.Material;
import com.jme3.math.Vector3f;
import com.jme3.scene.Geometry;
import com.jme3.scene.shape.Quad;
import com.jme3.texture.Texture;
import com.simsilica.lemur.Axis;
import com.simsilica.lemur.Button;
import com.simsilica.lemur.Container;
import com.simsilica.lemur.Label;
import com.simsilica.lemur.TextField;
import com.simsilica.lemur.component.SpringGridLayout;

import com.simsilica.lemur.style.ElementId;
import pp.monopoly.client.MonopolyApp;
import pp.monopoly.client.StartMenu;


/**
 * CreateGameMenu class represents the menu for creating a new game.
 */
public class CreateGameMenu {

    private final MonopolyApp app;
    private final Container menuContainer;
    private Geometry background;
    private Label serverStatusLabel;

    public CreateGameMenu(MonopolyApp app) {
        this.app = app;

        // Hintergrundbild laden und hinzufügen
        addBackgroundImage();

        // Hauptcontainer für das Menü
        menuContainer = new Container(new SpringGridLayout(Axis.Y, Axis.X));
        menuContainer.setPreferredSize(new Vector3f(600, 400, 0)); // Feste Größe des Containers

        // Titel
        Label title = menuContainer.addChild(new Label("Neues Spiel", new ElementId("header")));
        title.setFont(app.getAssetManager().loadFont("Interface/Fonts/Metropolis/Metropolis-Bold-42.fnt"));
        title.setFontSize(50);


        // Eingabefelder-Container
        Container inputContainer = menuContainer.addChild(new Container(new SpringGridLayout(Axis.Y, Axis.X)));
        inputContainer.setPreferredSize(new Vector3f(200, 150, 0)); // Eingabefelder nicht ganz so breit
        inputContainer.setLocalTranslation(20, 0, 0); // Abstand vom Rand

        inputContainer.addChild(new Label("Server-Adresse:"));
        TextField playerNameField = inputContainer.addChild(new TextField("localhost"));
        playerNameField.setPreferredWidth(400); // Breite des Textfelds

        inputContainer.addChild(new Label("Port:"));
        TextField serverAddressField = inputContainer.addChild(new TextField("42069"));
        serverAddressField.setPreferredWidth(400); // Breite des Textfelds

        // Button-Container
        Container buttonContainer = menuContainer.addChild(new Container(new SpringGridLayout(Axis.X, Axis.Y)));
        buttonContainer.setPreferredSize(new Vector3f(400, 50, 0));
        buttonContainer.setLocalTranslation(20, 0, 0); // Abstand vom Rand

        // "Abbrechen"-Button
        Button cancelButton = buttonContainer.addChild(new Button("Abbrechen"));
        cancelButton.setPreferredSize(new Vector3f(120, 40, 0));
        cancelButton.addClickCommands(source -> goBackToStartMenu());

        // "Selber hosten"-Button
        Button hostButton = buttonContainer.addChild(new Button("Selber hosten"));
        hostButton.setPreferredSize(new Vector3f(120, 40, 0));
        hostButton.addClickCommands(source -> app.getNetworkSupport().startServerAndJoin());

        // "Beitreten"-Button
        Button joinButton = buttonContainer.addChild(new Button("Beitreten"));
        joinButton.setPreferredSize(new Vector3f(120, 40, 0));
        joinButton.addClickCommands(source -> {
            try {
                String host = playerNameField.getText().trim();
                int port = Integer.parseInt(serverAddressField.getText().trim());
                app.getNetworkSupport().connectToServer(host, port);
        
                // LobbyMenu öffnen
                app.enqueue(() -> {
                    app.getGuiNode().detachAllChildren(); // Bestehende GUI entfernen
                    new LobbyMenu(app); // LobbyMenu erstellen
                });
            } catch (NumberFormatException e) {
                app.errorDialog("Port muss eine Zahl sein.");
            }
        });

        // Serverstatus-Label
        serverStatusLabel = menuContainer.addChild(new Label("Serverstatus: Noch nicht gestartet"));
        serverStatusLabel.setFontSize(24);

        // Zentrierung des Containers
        menuContainer.setLocalTranslation(
                (app.getCamera().getWidth() - menuContainer.getPreferredSize().x) / 2,
                (app.getCamera().getHeight() + menuContainer.getPreferredSize().y) / 2,
                1  // Höhere Z-Ebene für den Vordergrund
        );
        app.getInputManager().addMapping("OpenTestWorld", new com.jme3.input.controls.KeyTrigger(com.jme3.input.KeyInput.KEY_T));
        app.getInputManager().addListener(new com.jme3.input.controls.ActionListener() {
            @Override
            public void onAction(String name, boolean isPressed, float tpf) {
                if (name.equals("OpenTestWorld") && isPressed) {
                    app.startTestWorld(); // Öffnet die TestWorld
                }
            }
        }, "OpenTestWorld");

        // TODO später entfernen

        app.getInputManager().addMapping("OpenBuyCard", new com.jme3.input.controls.KeyTrigger(com.jme3.input.KeyInput.KEY_B));
        app.getInputManager().addListener(new com.jme3.input.controls.ActionListener() {
            @Override
            public void onAction(String name, boolean isPressed, float tpf) {
                if (name.equals("OpenBuyCard") && isPressed) {
                    app.startBuyCard(); // Öffnet die TestWorld
                }
            }
        }, "OpenBuyCard");

        app.getGuiNode().attachChild(menuContainer);
    }

    /**
     * Lädt das Hintergrundbild und fügt es als geometrische Ebene hinzu.
     */
    private void addBackgroundImage() {
        Texture backgroundImage = app.getAssetManager().loadTexture("Pictures/unibw-Bib2.png");
        Quad quad = new Quad(app.getCamera().getWidth(), app.getCamera().getHeight());
        background = new Geometry("Background", quad);
        Material backgroundMaterial = new Material(app.getAssetManager(), "Common/MatDefs/Misc/Unshaded.j3md");
        backgroundMaterial.setTexture("ColorMap", backgroundImage);
        background.setMaterial(backgroundMaterial);
        background.setLocalTranslation(0, 0, -1); // Hintergrundebene

        app.getGuiNode().attachChild(background);
    }

    /**
     * Geht zum Startmenü zurück, wenn "Abbrechen" angeklickt wird.
     */
    private void goBackToStartMenu() {
        app.getGuiNode().detachChild(menuContainer);
        app.getGuiNode().detachChild(background); // Entfernt das Hintergrundbild
        StartMenu.createStartMenu(app);
    }
}
