////////////////////////////////////////
// Programming project code
// UniBw M, 2022, 2023, 2024
// www.unibw.de/inf2
// (c) Mark Minas (mark.minas@unibw.de)
////////////////////////////////////////

package pp.monopoly.client.gui;

import com.jme3.material.Material;
import com.jme3.material.RenderState.BlendMode;
import com.jme3.math.ColorRGBA;
import com.jme3.renderer.queue.RenderQueue.ShadowMode;
import com.jme3.scene.Geometry;
import com.jme3.scene.Node;
import com.jme3.scene.Spatial;
import com.jme3.scene.shape.Box;

import pp.monopoly.client.MonopolyApp;
import pp.monopoly.game.server.PlayerColor;
import pp.monopoly.model.Board;
import pp.monopoly.model.Figure;
import pp.monopoly.model.Rotation;
import static pp.util.FloatMath.HALF_PI;
import static pp.util.FloatMath.PI;

/**
 * The {@code GameBoardSynchronizer} class is responsible for synchronizing the graphical
 * representation of the ships and shots on the sea map with the underlying data model.
 * It extends the {@link BoardSynchronizer} to provide specific synchronization
 * logic for the sea map.
 */
class GameBoardSynchronizer extends BoardSynchronizer {
    private static final String UNSHADED = "Common/MatDefs/Misc/Unshaded.j3md"; //NON-NLS
    private static final String LIGHTING = "Common/MatDefs/Light/Lighting.j3md";
    private static final String COLOR = "Color"; //NON-NLS
    private static final String FIGURE = "figure"; //NON-NLS

    private final MonopolyApp app;
    private final ParticleEffectFactory particleFactory;

    /**
     * Constructs a {@code GameBoardSynchronizer} object with the specified application, root node, and ship map.
     *
     * @param app  the Battleship application
     * @param root the root node to which graphical elements will be attached
     * @param map  the ship map containing the ships and shots
     */
    public GameBoardSynchronizer(MonopolyApp app, Node root, Board board) {
        super(board, root);
        this.app = app;
        this.particleFactory = new ParticleEffectFactory(app);
        addExisting();
    }

    /**
     * Visits a {@link Battleship} and creates a graphical representation of it.
     * The representation is either a 3D model or a simple box depending on the
     * type of battleship.
     *
     * @param ship the battleship to be represented
     * @return the node containing the graphical representation of the battleship
     */
    public Spatial visit(Figure figure) {
        final Node node = new Node(FIGURE);
        node.attachChild(createBox(figure));
        // compute the center of the ship in world coordinates
        final float x = 1;
        final float z = 1;
        node.setLocalTranslation(x, 0f, z);
        return node;
    }

    /**
     * Creates a simple box to represent a battleship that is not of the "King George V" type.
     *
     * @param ship the battleship to be represented
     * @return the geometry representing the battleship as a box
     */
    private Spatial createBox(Figure figure) {
        final Box box = new Box(0.5f * (figure.getMaxY() - figure.getMinY()) + 0.3f,
                                0.3f,
                                0.5f * (figure.getMaxX() - figure.getMinX()) + 0.3f);
        final Geometry geometry = new Geometry(FIGURE, box);
        geometry.setMaterial(createColoredMaterial(PlayerColor.PINK.getColor()));
        geometry.setShadowMode(ShadowMode.CastAndReceive);

        return geometry;
    }

    /**
     * Creates a new {@link Material} with the specified color.
     * If the color includes transparency (i.e., alpha value less than 1),
     * the material's render state is set to use alpha blending, allowing for
     * semi-transparent rendering.
     *
     * @param color the {@link ColorRGBA} to be applied to the material. If the alpha value
     *              of the color is less than 1, the material will support transparency.
     * @return a {@link Material} instance configured with the specified color and,
     * if necessary, alpha blending enabled.
     */
    private Material createColoredMaterial(ColorRGBA color) {
        final Material material = new Material(app.getAssetManager(), UNSHADED);
        if (color.getAlpha() < 1f)
            material.getAdditionalRenderState().setBlendMode(BlendMode.Alpha);
        material.setColor(COLOR, color);
        return material;
    }

    /**
     * Calculates the rotation angle for the specified rotation.
     *
     * @param rot the rotation of the battleship
     * @return the rotation angle in radians
     */
    private static float calculateRotationAngle(Rotation rot) {
        return switch (rot) {
            case RIGHT -> HALF_PI;
            case DOWN -> 0f;
            case LEFT -> -HALF_PI;
            case UP -> PI;
        };
    }
}
