package pp.monopoly.client.gui;

import com.jme3.material.Material;
import com.jme3.math.ColorRGBA;
import com.jme3.math.Vector3f;
import com.jme3.scene.Geometry;
import com.jme3.scene.shape.Box;
import com.jme3.texture.Texture;

import pp.monopoly.client.MonopolyApp;

/**
 * TestWorld zeigt eine einfache Szene mit einem texturierten Quadrat.
 * Die Kamera wird durch den CameraController gesteuert.
 */
public class TestWorld {

    private final MonopolyApp app;
    private CameraController cameraController; // Steuert die Kamera
    private Geometry cube; // Spielfigur

    /**
     * Konstruktor für TestWorld.
     *
     * @param app Die Hauptanwendung (MonopolyApp)
     */
    public TestWorld(MonopolyApp app) {
        this.app = app;
    }

    /**
     * Initialisiert die Szene und startet die Kamerabewegung.
     */
    public void initializeScene() {
        app.getGuiNode().detachAllChildren(); // Entferne GUI
        app.getRootNode().detachAllChildren(); // Entferne andere Szenenobjekte

        setSkyColor(); // Setze den Himmel auf hellblau
        createBoard(); // Erstelle das Spielfeld
        createCube();  // Füge den Würfel hinzu

        // Erstelle den CameraController
        cameraController = new CameraController(
                app.getCamera(),           // Die Kamera der App
                Vector3f.ZERO,            // Fokus auf die Mitte des Spielfelds
                5,                        // Radius des Kreises
                3,                        // Höhe der Kamera
                0.5f                      // Geschwindigkeit der Bewegung
        );

        // Füge die Toolbar hinzu
        new Toolbar(app, cube);
    }

    /**
     * Aktualisiert die Kameraposition.
     *
     * @param tpf Zeit pro Frame
     */
    public void update(float tpf) {
        if (cameraController != null) {
            cameraController.update(tpf);
        }
    }

    /**
     * Setzt die Hintergrundfarbe der Szene auf hellblau.
     */
    private void setSkyColor() {
        app.getViewPort().setBackgroundColor(new ColorRGBA(0.5f, 0.7f, 1.0f, 1.0f)); // Hellblauer Himmel
    }

    /**
     * Erstelle das Spielfeld.
     */
    private void createBoard() {
        // Erstelle ein Quadrat
        Box box = new Box(1, 0.01f, 1);  // Dünnes Quadrat für die Textur
        Geometry geom = new Geometry("Board", box);

        // Setze das Material mit Textur
        Material mat = new Material(app.getAssetManager(), "Common/MatDefs/Misc/Unshaded.j3md");
        Texture texture = app.getAssetManager().loadTexture("Pictures/board2.png");
        mat.setTexture("ColorMap", texture);
        geom.setMaterial(mat);

        app.getRootNode().attachChild(geom);
    }

    /**
     * Erstellt den Würfel (Spielfigur) in der Szene.
     */
    private void createCube() {
        Box box = new Box(0.05f, 0.05f, 0.05f); // Kleinere Größe für Spielfigur
        cube = new Geometry("Cube", box);

        // Setze das Material für den Würfel
        Material mat = new Material(app.getAssetManager(), "Common/MatDefs/Misc/Unshaded.j3md");
        mat.setColor("Color", ColorRGBA.Blue); // Blau gefärbter Würfel
        cube.setMaterial(mat);

        // Setze den Startpunkt des Würfels
        cube.setLocalTranslation(0.8999999f, 0.1f, -0.9f);

        app.getRootNode().attachChild(cube);
    }
}
