package pp.monopoly.game.client;

import java.lang.System.Logger;
import java.lang.System.Logger.Level;
import java.util.ArrayList;
import java.util.List;

import pp.monopoly.message.client.ClientMessage;
import pp.monopoly.message.server.BuyPropertyResponse;
import pp.monopoly.message.server.DiceResult;
import pp.monopoly.message.server.EventDrawCard;
import pp.monopoly.message.server.GameOver;
import pp.monopoly.message.server.GameStart;
import pp.monopoly.message.server.JailEvent;
import pp.monopoly.message.server.NextPlayerTurn;
import pp.monopoly.message.server.PlayerStatusUpdate;
import pp.monopoly.message.server.ServerInterpreter;
import pp.monopoly.message.server.TimeOutWarning;
import pp.monopoly.message.server.TradeReply;
import pp.monopoly.message.server.TradeRequest;
import pp.monopoly.message.server.ViewAssetsResponse;
import pp.monopoly.model.Board;
import pp.monopoly.model.IntPoint;
import pp.monopoly.notification.ClientStateEvent;
import pp.monopoly.notification.GameEvent;
import pp.monopoly.notification.GameEventBroker;
import pp.monopoly.notification.GameEventListener;
import pp.monopoly.notification.InfoTextEvent;
import pp.monopoly.notification.Sound;
import pp.monopoly.notification.SoundEvent;

/**
 * Controls the client-side game logic for Monopoly.
 * Handles interactions with the server and game state management on the client side.
 */
public class ClientGameLogic implements ServerInterpreter, GameEventBroker {

    /** Logger for the client-side game logic. */
    static final Logger LOGGER = System.getLogger(ClientGameLogic.class.getName());

    /** The object responsible for sending messages to the server. */
    private final ClientSender clientSender;

    /** A list of listeners to receive game events. */
    private final List<GameEventListener> listeners = new ArrayList<>();

    /** The game board representing the player's current state. */
    private Board board;

    /** The current state of the client game logic. */
    private ClientState state = new LobbyState(this);

    /**
     * Constructs a ClientGameLogic with the specified sender object.
     *
     * @param clientSender the object used to send messages to the server
     */
    public ClientGameLogic(ClientSender clientSender) {
        this.clientSender = clientSender;
    }

    /**
     * Returns the current state of the game logic.
     *
     * @return the current state
     */
    ClientState getState() {
        return state;
    }

    /**
     * Sets the current state of the game logic.
     *
     * @param newState the new state to be set
     */
    void setState(ClientState newState) {
        LOGGER.log(Level.DEBUG, "state transition {0} --> {1}", state.getName(), newState.getName()); //NON-NLS
        state = newState;
        notifyListeners(new ClientStateEvent());
        state.entry();
    }

    /**
     * Returns the player's game board.
     *
     * @return the player's game board
     */
    public Board getBoard() {
        return board;
    }

    /**
     * Moves the preview figure to the specified position.
     *
     * @param pos the new position for the preview figure
     */
    public void movePreview(IntPoint pos) {
        state.movePreview(pos);
    }

    /**
     * Sets the informational text to be displayed to the player.
     *
     * @param key the key for the info text
     */
    void setInfoText(String key) {
        notifyListeners(new InfoTextEvent(key));
    }

    /**
     * Emits an event to play the specified sound.
     *
     * @param sound the sound to be played
     */
    public void playSound(Sound sound) {
        notifyListeners(new SoundEvent(sound));
    }

    /**
     * Sends a message to the server.
     *
     * @param msg the message to be sent
     */
    void send(ClientMessage msg) {
        if (clientSender == null) {
            LOGGER.log(Level.ERROR, "trying to send {0} with sender==null", msg); //NON-NLS
        } else {
            clientSender.send(msg);
        }
    }

    /**
     * Adds a listener to receive game events.
     *
     * @param listener the listener to add
     */
    public synchronized void addListener(GameEventListener listener) {
        listeners.add(listener);
    }

    /**
     * Removes a listener from receiving game events.
     *
     * @param listener the listener to remove
     */
    public synchronized void removeListener(GameEventListener listener) {
        listeners.remove(listener);
    }

    /**
     * Notifies all listeners of a game event.
     *
     * @param event the game event to notify listeners of
     */
    @Override
    public void notifyListeners(GameEvent event) {
        final List<GameEventListener> copy;
        synchronized (this) {
            copy = new ArrayList<>(listeners);
        }
        for (GameEventListener listener : copy) {
            event.notifyListener(listener);
        }
    }

    /**
     * Updates the game logic once per frame in the update loop.
     *
     * @param delta time in seconds since the last update call
     */
    public void update(float delta) {
        state.update(delta);
    }

    /**
     * Handles the response for buying a property.
     *
     * @param msg the message containing the buy property response
     */
    @Override
    public void received(BuyPropertyResponse msg) {
        if (msg.isSuccessful()) {
            setInfoText("You successfully bought " + msg.getPropertyName() + "!");
            playSound(Sound.MONEY_LOST);
        } else {
            setInfoText("Unable to buy " + msg.getPropertyName() + ". Reason: " + msg.getReason());
        }
    }

    /**
     * Handles the result of a dice roll.
     *
     * @param msg the message containing the dice roll result
     */
    @Override
    public void received(DiceResult msg) {
        setInfoText("You rolled a " + msg.calcTotal() + "!");
        //Set the dice images
        playSound(Sound.DICE_ROLL);
    }

    /**
     * Handles drawing an event card.
     *
     * @param msg the message containing the drawn card details
     */
    @Override
    public void received(EventDrawCard msg) {
        setInfoText("Event card drawn: " + msg.getCardDescription());
        //event card logic
        playSound(Sound.EVENT_CARD);
    }

    /**
     * Handles the game over message.
     *
     * @param msg the message containing game over details
     */
    @Override
    public void received(GameOver msg) {
        if (msg.isWinner()) {
            setInfoText("Congratulations! You have won the game!");
            //Winner popup
            playSound(Sound.WINNER);
        } else {
            setInfoText("Game over. Better luck next time!");
            // Looser popup
            playSound(Sound.LOSER);
        }
    }

    /**
     * Handles the start of the game.
     *
     * @param msg the game start message
     */
    @Override
    public void received(GameStart msg) {
        setInfoText("The game has started! Good luck!");
        setState(new WaitForTurnState(this));
    }

    /**
     * Handles jail-related events.
     *
     * @param msg the message containing jail event details
     */
    @Override
    public void received(JailEvent msg) {
        if (msg.isGoingToJail()) {
            setInfoText("You are sent to jail!");
            playSound(Sound.GULAG);
        } else {
            setInfoText("You are out of jail!");
        }
    }

    /**
     * Updates the status of a player.
     *
     * @param msg the message containing player status update details
     */
    @Override
    public void received(PlayerStatusUpdate msg) {
        
        setInfoText("Player " + msg.getPlayerName() + " status updated: " + msg.getStatus());
    }

    /**
     * Handles timeout warnings.
     *
     * @param msg the message containing timeout warning details
     */
    @Override
    public void received(TimeOutWarning msg) {
        setInfoText("Warning! Time is running out. You have " + msg.getRemainingTime() + " seconds left.");
    }

    /**
     * Displays the player's assets in response to a server query.
     *
     * @param msg the message containing the player's assets
     */
    @Override
    public void received(ViewAssetsResponse msg) {
        setInfoText("Your current assets are being displayed.");
    }

    /**
     * Handles trade replies from other players.
     *
     * @param msg the message containing the trade reply
     */
    @Override
    public void received(TradeReply msg) {
        if (msg.getTradeHandler().getStatus()) {
            setInfoText("Trade accepted by " + msg.getTradeHandler().getReceiver().getName() + ".");
            playSound(Sound.TRADE_ACCEPTED);
        } else {
            setInfoText("Trade rejected by " + msg.getTradeHandler().getReceiver().getName() + ".");
            playSound(Sound.TRADE_REJECTED);
        }
    }

    /**
     * Handles trade requests from other players.
     *
     * @param msg the message containing the trade request details
     */
    @Override
    public void received(TradeRequest msg) {
        setInfoText("Trade offer received from " + msg.getTradeHandler().getSender().getName());
        // playSound(Sound.TRADE_REQUEST); no sound effect
        // notifyListeners();
    }

    /**
     * Handles the transition to the next player's turn.
     *
     * @param msg the message indicating it's the next player's turn
     */
    @Override
    public void received(NextPlayerTurn msg) {
        setInfoText("It's your turn!");
        setState(new ActiveState(this));
    }
}
