package pp.monopoly.model;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import static java.lang.Math.max;
import static java.lang.Math.min;

public class Figure implements Item{
    private final String type;
    private final int length; // The length of the Figure
    private int x;            // The x-coordinate of the Figure's position
    private int y;            // The y-coordinate of the Figure's position
    private Rotation rot;     // The rotation of the Figure
    private final Set<IntPoint> damaged = new HashSet<>(); // The set of positions that have been hit on this ship

    /**
     * Default constructor for serialization. Initializes a Figure with length 0,
     * at position (0, 0), with a default rotation of RIGHT.
     */
    private Figure() {
        this(0, 0, 0, Rotation.RIGHT, "cube");
    }

    /**
     * Constructs a new Figure with the specified length, position, and rotation.
     *
     * @param length the length of the Figure
     * @param x      the x-coordinate of the Figure's initial position
     * @param y      the y-coordinate of the Figure's initial position
     * @param rot    the rotation of the Figure
     */
    public Figure(int length, int x, int y, Rotation rot, String type) {
        this.x = x;
        this.y = y;
        this.rot = rot;
        this.length = length;
        this.type = type;
    }

    /**
     * Returns the current x-coordinate of the Figure's position.
     *
     * @return the x-coordinate of the Figure
     */
    public int getX() {
        return x;
    }

    /**
     * Returns the current y-coordinate of the Figure's position.
     *
     * @return the y-coordinate of the Figure
     */
    public int getY() {
        return y;
    }

    /**
     * Moves the Figure to the specified coordinates.
     *
     * @param x the new x-coordinate of the Figure's position
     * @param y the new y-coordinate of the Figure's position
     */
    public void moveTo(int x, int y) {
        this.x = x;
        this.y = y;
    }
    
    /**
     * Moves the Figure to the specified position.
     *
     * @param pos the new position of the Figure
     */
    public void moveTo(IntPosition pos) {
        moveTo(pos.getX(), pos.getY());
    }

    /**
     * Moves the Figure to the specified coordinates.
     *
     * @param x the new x-coordinate of the Figure's position
     * @param y the new y-coordinate of the Figure's position
     */
    public void moveTo(int fieldId) {
        moveTo(fieldIdToPosition(fieldId));
    }

    private IntPoint fieldIdToPosition(int fieldId) {
        if (fieldId < 0 || fieldId > 39) {
            throw new IllegalArgumentException("Invalid fieldId: " + fieldId);
        }

        // Determine which edge and position along the edge
        if (fieldId <= 9) {
            // Bottom edge: From (-10, -10) to (10, -10)
            int x = -10 + fieldId * 2;
            return new IntPoint(x, -10);
        } else if (fieldId <= 19) {
            // Right edge: From (10, -10) to (10, 10)
            int y = -10 + (fieldId - 10) * 2;
            return new IntPoint(10, y);
        } else if (fieldId <= 29) {
            // Top edge: From (10, 10) to (-10, 10)
            int x = 10 - (fieldId - 20) * 2;
            return new IntPoint(x, 10);
        } else {
            // Left edge: From (-10, 10) to (-10, -10)
            int y = 10 - (fieldId - 30) * 2;
            return new IntPoint(-10, y);
        }
    }

    private Rotation fieldIdToRotation(int fieldId) {
        if (fieldId >= 0 && fieldId <= 10) return Rotation.DOWN;
        else if (fieldId <= 20) return Rotation.LEFT;
        else if (fieldId <= 30) return Rotation.UP;
        else if (fieldId <= 39) return Rotation.RIGHT;
        else throw new IllegalArgumentException();
    }

    /**
     * Returns the length of the Figure.
     *
     * @return the length of the Figure
     */
    public int getLength() {
        return length;
    }

    /**
     * Returns the minimum x-coordinate that the Figure occupies based on its current position and rotation.
     *
     * @return the minimum x-coordinate of the Figure
     */
    public int getMinX() {
        return x + min(0, (length - 1) * rot.dx());
    }

    /**
     * Returns the maximum x-coordinate that the Figure occupies based on its current position and rotation.
     *
     * @return the maximum x-coordinate of the Figure
     */
    public int getMaxX() {
        return x + max(0, (length - 1) * rot.dx());
    }

    /**
     * Returns the minimum y-coordinate that the Figure occupies based on its current position and rotation.
     *
     * @return the minimum y-coordinate of the Figure
     */
    public int getMinY() {
        return y + min(0, (length - 1) * rot.dy());
    }

    /**
     * Returns the maximum y-coordinate that the Figure occupies based on its current position and rotation.
     *
     * @return the maximum y-coordinate of the Figure
     */
    public int getMaxY() {
        return y + max(0, (length - 1) * rot.dy());
    }

    /**
     * Returns the current rotation of the Figure.
     *
     * @return the rotation of the Figure
     */
    public Rotation getRot() {
        return rot;
    }

    /**
     * Sets the rotation of the Figure.
     *
     * @param rot the new rotation to be set for the Figure
     */
    public void setRotation(Rotation rot) {
        this.rot = rot;
    }

    /**
     * Rotates the Figure by 90 degrees clockwise.
     */
    public void rotated() {
        setRotation(rot.rotate());
    }

    /**
     * Attempts to hit the Figure at the specified position.
     * If the position is part of the Figure, the hit is recorded.
     *
     * @param x the x-coordinate of the position to hit
     * @param y the y-coordinate of the position to hit
     * @return true if the position is part of the Figure, false otherwise
     * @see #contains(int, int)
     */
    public boolean hit(int x, int y) {
        if (!contains(x, y))
            return false;
        damaged.add(new IntPoint(x, y));
        return true;
    }

    /**
     * Attempts to hit the Figure at the specified position.
     * If the position is part of the Figure, the hit is recorded.
     * This is a convenience method for {@linkplain #hit(int, int)}.
     *
     * @param position the position to hit
     * @return true if the position is part of the Figure, false otherwise
     */
    public boolean hit(IntPosition position) {
        return hit(position.getX(), position.getY());
    }

    /**
     * Returns the positions of this Figure that have been hit.
     *
     * @return a set of positions that have been hit
     * @see #hit(int, int)
     */
    public Set<IntPoint> getDamaged() {
        return Collections.unmodifiableSet(damaged);
    }

    /**
     * Checks whether the specified position is covered by the Figure. This method does
     * not record a hit, only checks coverage.
     * This is a convenience method for {@linkplain #contains(int, int)}.
     *
     * @param pos the position to check
     * @return true if the position is covered by the Figure, false otherwise
     */
    public boolean contains(IntPosition pos) {
        return contains(pos.getX(), pos.getY());
    }

    /**
     * Checks whether the specified position is covered by the Figure. This method does
     * not record a hit, only checks coverage.
     *
     * @param x the x-coordinate of the position to check
     * @param y the y-coordinate of the position to check
     * @return true if the position is covered by the Figure, false otherwise
     */
    public boolean contains(int x, int y) {
        return getMinX() <= x && x <= getMaxX() &&
               getMinY() <= y && y <= getMaxY();
    }

    /**
     * Determines if the Figure has been completely destroyed. A Figure is considered
     * destroyed if all of its positions have been hit.
     *
     * @return true if the Figure is destroyed, false otherwise
     * @see #hit(int, int)
     */
    public boolean isDestroyed() {
        return damaged.size() == length;
    }

    /**
     * Checks whether this Figure collides with another Figure. Two Figures collide
     * if any of their occupied positions overlap.
     *
     * @param other the other Figure to check collision with
     * @return true if the Figures collide, false otherwise
     */
    public boolean collidesWith(Figure other) {
        return other.getMaxX() >= getMinX() && getMaxX() >= other.getMinX() &&
               other.getMaxY() >= getMinY() && getMaxY() >= other.getMinY();
    }

    /**
     * Returns a string representation of the Figure, including its length, position,
     * and rotation.
     *
     * @return a string representation of the Figure
     */
    @Override
    public String toString() {
        return "Ship{length=" + length + ", x=" + x + ", y=" + y + ", rot=" + rot + '}';  //NON-NLS
    }

    /**
     * Accepts a visitor that returns a value of type {@code T}. This method is part of the
     * Visitor design pattern.
     *
     * @param visitor the visitor to accept
     * @param <T>     the type of the value returned by the visitor
     * @return the value returned by the visitor
     */
    @Override
    public <T> T accept(Visitor<T> visitor) {
        return visitor.visit(this);
    }

    /**
     * Accepts a visitor that does not return a value. This method is part of the
     * Visitor design pattern.
     *
     * @param visitor the visitor to accept
     */
    @Override
    public void accept(VoidVisitor visitor) {
        visitor.visit(this);
    }

}
