package pp.monopoly.model;

import java.util.LinkedList;

/**
 * A LinkedList with a maximum size limit.
 *
 * @param <E> the type of elements held in this collection
 */
public class LimitedLinkedList<E> extends LinkedList<E> {

    private final int maxSize;

    /**
     * Constructs a LimitedLinkedList with the specified maximum size.
     *
     * @param maxSize the maximum number of elements this list can hold
     */
    public LimitedLinkedList(int maxSize) {
        if (maxSize <= 0) {
            throw new IllegalArgumentException("Max size must be greater than 0");
        }
        this.maxSize = maxSize;
    }

    /**
     * Adds an element to the list. If the list exceeds its maximum size,
     * the oldest element (first) is removed.
     *
     * @param element the element to be added
     * @return true if the element was added successfully
     */
    @Override
    public boolean add(E element) {
        if (size() >= maxSize) {
            return false;
        }
        return super.add(element);
    }

    /**
     * Gets the maximum size of this list.
     *
     * @return the maximum size
     */
    public int getMaxSize() {
        return maxSize;
    }
}
