package pp.monopoly.model;

import pp.monopoly.game.server.Player;
import pp.monopoly.model.fields.PropertyField;

import java.util.List;

/**
 * Handles a single trade between two players.
 * Encapsulates trade details, validation, acceptance, and rejection.
 */
public class TradeHandler {

    private final Player sender;
    private final Player receiver;
    private final int offeredAmount;
    private final List<PropertyField> offeredProperties;
    private final int offeredJailCards;
    private final int requestedAmount;
    private final List<PropertyField> requestedProperties;
    private final int requestedJailCards;
    private Boolean status = null;

    /**
     * Constructs a TradeHandler for a single trade instance.
     *
     * @param sender             the Player initiating the trade
     * @param receiver           the Player receiving the trade offer
     * @param offeredAmount      the amount of money offered by the sender
     * @param offeredProperties  the properties offered by the sender
     * @param offeredJailCards   the jail cards offered by the sender
     * @param requestedAmount    the amount of money requested from the receiver
     * @param requestedProperties the properties requested from the receiver
     * @param requestedJailCards the jail cards requested from the receiver
     */
    public TradeHandler(Player sender, Player receiver, int offeredAmount, List<PropertyField> offeredProperties,
                        int offeredJailCards, int requestedAmount, List<PropertyField> requestedProperties, int requestedJailCards) {
        this.sender = sender;
        this.receiver = receiver;
        this.offeredAmount = offeredAmount;
        this.offeredProperties = offeredProperties;
        this.offeredJailCards = offeredJailCards;
        this.requestedAmount = requestedAmount;
        this.requestedProperties = requestedProperties;
        this.requestedJailCards = requestedJailCards;
    }

    public int getOfferedAmount() {
        return offeredAmount;
    }

    public int getOfferedJailCards() {
        return offeredJailCards;
    }

    public List<PropertyField> getOfferedProperties() {
        return offeredProperties;
    }

    public Player getReceiver() {
        return receiver;
    }

    public int getRequestedAmount() {
        return requestedAmount;
    }

    public int getRequestedJailCards() {
        return requestedJailCards;
    }

    public List<PropertyField> getRequestedProperties() {
        return requestedProperties;
    }

    public Player getSender() {
        return sender;
    }

    public Boolean getStatus() {
        return status;
    }

    /**
     * Initiates the trade and validates its terms.
     *
     * @return true if the trade is valid and can proceed, false otherwise
     */
    public boolean initiateTrade() {
        if (!validateTrade()) {
            System.out.println("Trade offer is invalid.");
            return false;
        }
        System.out.println("Trade initiated by " + sender.getName() + " to " + receiver.getName());
        return true;
    }

    /**
     * Completes the trade by transferring money, properties, and jail cards.
     */
    public void acceptTrade() {
        // Transfer money
        sender.earnMoney(-offeredAmount);
        receiver.earnMoney(offeredAmount);

        receiver.earnMoney(-requestedAmount);
        sender.earnMoney(requestedAmount);

        // Transfer properties
        if (offeredProperties != null) {
            for (PropertyField property : offeredProperties) {
                transferProperty(sender, receiver, property);
            }
        }
        if (requestedProperties != null) {
            for (PropertyField property : requestedProperties) {
                transferProperty(receiver, sender, property);
            }
        }

        // Transfer jail cards
        transferJailCards(sender, receiver, offeredJailCards);
        transferJailCards(receiver, sender, requestedJailCards);

        System.out.println("Trade completed between " + sender.getName() + " and " + receiver.getName());
    }

    /**
     * Rejects the trade.
     */
    public void rejectTrade() {
        System.out.println(receiver.getName() + " rejected the trade.");
    }

    /**
     * Validates the trade offer by checking ownership, balances, and jail cards.
     *
     * @return true if the trade is valid, false otherwise
     */
    private boolean validateTrade() {
        // Validate sender's ability to offer money
        if (sender.getAccountBalance() < offeredAmount) {
            System.out.println("Sender does not have enough money to offer.");
            return false;
        }

        // Validate receiver's ability to fulfill the requested amount
        if (receiver.getAccountBalance() < requestedAmount) {
            System.out.println("Receiver does not have enough money to fulfill the request.");
            return false;
        }

        // Validate property ownership
        if (offeredProperties != null) {
            for (PropertyField property : offeredProperties) {
                if (!sender.getProperties().contains(property)) {
                    System.out.println("Sender does not own property: " + property.getName());
                    return false;
                }
            }
        }
        if (requestedProperties != null) {
            for (PropertyField property : requestedProperties) {
                if (!receiver.getProperties().contains(property)) {
                    System.out.println("Receiver does not own property: " + property.getName());
                    return false;
                }
            }
        }

        // Validate jail cards
        if (sender.getNumJailCard() < offeredJailCards) {
            System.out.println("Sender does not have enough jail cards to offer.");
            return false;
        }
        if (receiver.getNumJailCard() < requestedJailCards) {
            System.out.println("Receiver does not have enough jail cards to fulfill the request.");
            return false;
        }

        return true;
    }

    /**
     * Transfers a property between players.
     *
     * @param from the Player transferring the property
     * @param to   the Player receiving the property
     * @param property the PropertyField being transferred
     */
    private void transferProperty(Player from, Player to, PropertyField property) {
        from.sellProperty(property);
        to.buyProperty(property);
        property.setOwner(to);
        System.out.println("Property " + property.getName() + " transferred from " + from.getName() + " to " + to.getName());
    }

    /**
     * Transfers jail cards between players.
     *
     * @param from the Player transferring jail cards
     * @param to   the Player receiving jail cards
     * @param numCards the number of jail cards to transfer
     */
    private void transferJailCards(Player from, Player to, int numCards) {
        for (int i = 0; i < numCards; i++) {
            from.removeJailCard();
            to.addJailCard();
        }
        System.out.println(numCards + " jail card(s) transferred from " + from.getName() + " to " + to.getName());
    }
}
